library(rstan)
library(sp)
library(tibble)
library(ggplot2)
library(MappingUtilities)
library(BMNUS)
library(Matrix)

Add_Geography <- function(boundary_color = "gray70") {

  map_df <- ggplot2::map_data("state", region = c("Virginia", "North Carolina",
                                                  "South Carolina", "Georgia",
                                                  "Alabama", "Florida"))

  text_df <- data.frame(x = c(-86.7, -84, -82, -81, -80, -81.7, -78, - 86),
                        y = c(34.2, 34.2, 34.8, 35.9, 37, 28.5, 31, 28.5),
                        label = c("Alabama", "Georgia",
                                  "South\nCarolina", "North\nCarolina",
                                  "Virginia", "Florida", "Atlantic Ocean",
                                  "Gulf of Mexico"))
  return(
    list(ggplot2::geom_path(ggplot2::aes(x = long, y = lat, group = group),
                            data = map_df,
                            colour = boundary_color),
         ggplot2::geom_text(ggplot2::aes(x = x, y = y, label = label),
                            size = 2.5,
                            data = text_df)))
}


directory <- "..\\..\\UsersGuide_Revised\\ReportScripts\\"

load(paste0(directory, "SamplingResults\\RawSamples.dat"))
load(paste0(directory, "BasisFunctions\\Spacing_030.dat"))
load(paste0(directory, "Predictions\\PredictedValues.dat"))

##
## Compute the covariance associated with the mean of the process model.
##

postPdfSamples <- extract(rawSamples)

nSamples <- length(postPdfSamples$tau1)
M <- stanData$M

# W is Cov(phi1, phi1)
W <- matrix(0.0, nrow = M, ncol = M)
mean_phi1 <- apply(postPdfSamples$phi1, 2, mean)
for(i in 1:nSamples){
  a <- as.vector(postPdfSamples$phi1[i, ] - mean_phi1)
  W <- W + a %o% a
}
W <- W / nSamples

# Bp is the matrix of basis function values for all prediction points
Bp <- BasisFuncMatrix(basis_functions$centers_utm,
                      predictedValues[, c("easting", "northing")],
                      basis_functions$sepDistance,
                      as.integer(1e7))

# Z is Cov(Y1, Y1) where Y1 represents the mean of the process model
# Z contains the covariance for every prediction point (in the domain)
# See equation 3.3 in the manuscript
Z <- Bp %*% W %*% t(Bp)

##
## Decompose the covariance matrix (Z) into a correlation matrix (K) and
## two, identical diagonal matrices (D_sparse). Only the diagonal of D_sparse
## is stored. The elements along the diagonal are the standard deviations.
## That is, the decomposition is
##     Z = diag(D_sparse) %*% K %*% diag(D_sparse)
##

D_sparse <- sqrt(diag(Z))

tmp <- D_sparse %o% D_sparse

K <- Z / tmp

##
## Map the standard deviations associated with the process mean
##

df_stdDev <- data.frame(
  long = predictedValues$long,
  lat = predictedValues$lat,
  stdDev = D_sparse
)

plotPalette <- colorRampPalette(c("blue", "green",
                                  "yellow", "orange", "red", "black"))(12)

values <- NULL

postscript("..\\Figures\\Figure_3-1.ps", width = 4, height = 4)
ggplot() +
  Add_Marked_Points(df_stdDev, "stdDev", "Std Dev", values, colors = plotPalette) +
  Add_Path(Example_roi, "black") +
  Add_Geography(boundary_color = "gray50") +
  Refine_Map("lambert", 25, 40, latLimits = c(28, 39.5)) +
  Refine_Legend(c(0.02,0.99), c(0,1), legend.direction = "horizontal") +
  theme(plot.title=element_text(hjust = 0, face = "bold.italic", size = 10))
dev.off()

##
## Determine the points at which the correlation function is analyzed.
## These points are the prediction points that are closest to the points used
## for the sensitivity analysis. These points are called reference points.
##

# locations of basis functions centers that are used for the sensitivity analysis
df_points <- as_tibble(basis_functions$centers_utm)[c(293, 110, 260, 456), ]
df_points$label <- c("1", "2", "3", "4")
colnames(df_points) <- c("easting", "northing", "label")

FindClosestPoint <- function(point, locations){
  d2 <- (point$easting - locations$easting)^2 +
    (point$northing - locations$northing)^2

  return(which.min(d2))
}

indices <- c(FindClosestPoint(df_points[1, ], predictedValues),
             FindClosestPoint(df_points[2, ], predictedValues),
             FindClosestPoint(df_points[3, ], predictedValues),
             FindClosestPoint(df_points[4, ], predictedValues))

reference_points <- tibble(index = indices,
                           easting = predictedValues$easting[indices],
                           northing = predictedValues$northing[indices],
                           long = predictedValues$long[indices],
                           lat = predictedValues$lat[indices],
                           label = 1:4)

# Plot the reference points
ggplot() +
  Add_Geography(boundary_color = "gray50") +
  Add_Path(Example_domain, "blue") +
  Add_Points(reference_points, shape = 20, fill = "black", size = 3) +
  geom_label(aes(x = long, y = lat, label = label),
             data = reference_points,
             vjust = "top", color = "blue", nudge_y = -0.2) +
  Refine_Map("lambert", 25, 40, latLimits = c(28, 39.5))

##
## Map, for point 2, the correlation associated with the mean of the process model
##

cor_point2 <- data.frame(
  long = predictedValues$long,
  lat = predictedValues$lat,
  correlation = K[, reference_points$index[2]]
)

plotPalette <- colorRampPalette(c("gray", "red"))(20)

postscript("..\\Figures\\Figure_3-2.ps", width = 4, height = 4)
ggplot() +
  Add_Marked_Points(cor_point2, "correlation", "Correlation", NULL,
                    colors = plotPalette) +
  # Add_Points(reference_points, shape = 20, fill = "black", size = 1) +
  Add_Geography(boundary_color = "gray50") +
  Add_Path(Example_domain, "blue") +
  Refine_Map("lambert", 25, 40, latLimits = c(28, 39.5)) +
  Refine_Legend(c(0.02,0.99), c(0,1),
                legend.direction = "horizontal") +
  theme(plot.title=element_text(hjust = 0, face = "bold.italic", size = 10))
dev.off()


##
## Plot the correlation function along transects that pass through the
## four reference points.
##

areInTransect1 <- (predictedValues$easting == reference_points$easting[1]) &
  abs(predictedValues$northing - reference_points$northing[1]) <= 100000

areInTransect2 <- (predictedValues$easting == reference_points$easting[2]) &
  abs(predictedValues$northing - reference_points$northing[2]) <= 100000

areInTransect3 <- (predictedValues$easting == reference_points$easting[3]) &
  abs(predictedValues$northing - reference_points$northing[3]) <= 100000

areInTransect4 <- (predictedValues$easting == reference_points$easting[4]) &
  abs(predictedValues$northing - reference_points$northing[4]) <= 100000

# check
ggplot() +
  Add_Points(predictedValues[areInTransect1, ]) +
  Add_Points(predictedValues[areInTransect2, ]) +
  Add_Points(predictedValues[areInTransect3, ]) +
  Add_Points(predictedValues[areInTransect4, ]) +
  Add_Points(reference_points, shape = 20, fill = "red", size = 2) +
  Add_Geography(boundary_color = "gray50") +
  Add_Path(Example_domain, "blue") +
  Refine_Map("lambert", 25, 40, latLimits = c(28, 39.5)) +
  Refine_Legend(c(0.02,0.99), c(0,1),
                legend.direction = "horizontal") +
  theme(plot.title=element_text(hjust = 0, face = "bold.italic", size = 10))

# Generate tibbles with correlation along the transects
df1 <- tibble(distance = (predictedValues$northing[areInTransect1] - reference_points$northing[1]) / 1000,
              value = K[areInTransect1, reference_points$index[1]],
              point = 1)

df2 <- tibble(distance = (predictedValues$northing[areInTransect2] - reference_points$northing[2]) / 1000,
              value = K[areInTransect2, reference_points$index[2]],
              point = 2)

df3 <- tibble(distance = (predictedValues$northing[areInTransect3] - reference_points$northing[3]) / 1000,
              value = K[areInTransect3, reference_points$index[3]],
              point = 3)

df4 <- tibble(distance = (predictedValues$northing[areInTransect4] - reference_points$northing[4]) / 1000,
              value = K[areInTransect4, reference_points$index[4]],
              point = 4)

df <- rbind(df1, df2, df3, df4)

postscript("..\\Figures\\Figure_3-3.ps", width = 3, height = 2)
ggplot() +
  geom_line(aes(x = distance, y = value, color = factor(point)),
            data = df) +
  scale_x_continuous(name = "Distance along transect, in kilometers") +
  scale_y_continuous(name = "Correlation") +
  labs(color = "Points") +
  theme_bw() +
  theme(plot.title=element_text(hjust = 0,face = "bold.italic", size = 10)) +
  theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank()) +
  theme(legend.justification = c(1,1), legend.position = c(1,1),
        legend.background = element_rect(fill = "white", color = "black"))
dev.off()

