##
## This file and file SensitivityPlots.R lists the R scripts that are used 
## for the sensitivity analysis.
##

library(BMNUS)
library(sp)
library(Matrix)
library(rstan)

##
## Organize the Spatial Data
##

# Transform the titanium concentrations using the isometric log-ratio
# transform
Value <- scaledLogit(Example_dataset$conc, constSumValue = 1e6)

# Store the transformed concentrations, the associated censor indicators in
# spatial points dataframe.
spatialData <-
  sp::SpatialPointsDataFrame(data.frame(long = Example_dataset$long,
                                        lat = Example_dataset$lat,
                                        row.names = Example_dataset$labno),
                             data.frame(Value = Value,
                                        IndValue = "no",
                                        row.names = Example_dataset$labno),
                             proj4string=sp::CRS(Example_CRS_arg_longlat),
                             match.ID=TRUE)

rm("Value")

##
## Generate the Basis Functions
##

# Construct a class that generates the basis functions
oBasisFunctions <- BasisFunctions("030",
                                  Example_domain,
                                  spatialData,
                                  Example_CRS_arg_utm,
                                  seed = 777)

rm("oBasisFunctions")

##
## Sample the Posterior Probability Density Function
##

# Determine the file name (including the complete path) for
# Stan program BMNUS
filename <- paste0(normalizePath(path.package("BMNUS")), "\\stan\\BMNUS.stan")

# Translate Stan program BMNUS into C++ functions and then compile
# these functions
tr <- stanc(file = filename, model_name = "BMNUS")
BMNUS_sm <- stan_model(stanc_ret = tr, verbose=FALSE)

# Prepare the data
load("BasisFunctions\\Spacing_030.dat")

csrBasisFuncs <- as(basis_functions$basisFuncs, "RsparseMatrix")

adjMatrix <- CalcAdjacencyMatrix(basis_functions)
stanData <- list( N = nrow(spatialData@data),
                  M = ncol(basis_functions$basisFuncs),
                  UU = csrBasisFuncs@p + 1,
                  VV = csrBasisFuncs@j + 1,
                  WW = csrBasisFuncs@x,
                  W = adjMatrix,
                  W_n = CalcNumberPairs(adjMatrix),
                  X = spatialData@data$Value,
                  me_sd = 0.032,
                  areLeftCensored = as.integer(spatialData@data[, "IndValue"] == "left"),
                  betaPar = c(2.5, 1.2),
                  gammaPar = c(2, 0.3),
                  cauchyPar = 3)

# Function gen_int initializes some variables for the sampling
gen_init <- function(){
  phi1 <- rnorm(stanData$M, mean = 0.0, sd = 1e-6)
  phi2 <- rnorm(stanData$M, mean = 0.0, sd = 1e-6)
  return(list(phi1 = phi1, phi2 = phi2))
}

nChains <- 3
if(nChains < parallel::detectCores()){
  oldPar <- options(mc.cores = nChains)
} else{
  oldPar <- options(mc.cores = parallel::detectCores()-1)
}


# Sample the posterior pdf
rawSamples <- sampling(BMNUS_sm, data = stanData, chains = nChains,
                       iter = 2000, warmup = 500,
                       refresh = 100,
                       seed = 7,
                       init = gen_init,
                       pars=c("Y_mean", "Y_sd"))

# Archive the samples for subsequent analysis
if(!dir.exists("SamplingResults")){
  dir.create("SamplingResults")
}

save(stanData, rawSamples, file = "SamplingResults\\RawSamples.dat")

rm("filename", "tr", "basis_functions", "csrBasisFuncs", "adjMatrix", 
   "spatialData", "rawSamples")

##
## Change betaPar[1]
## 

stanData$betaPar[1] <- 5.0

rawSamples <- sampling(BMNUS_sm, data = stanData, chains = nChains,
                       iter = 2000, warmup = 500,
                       refresh = 100,
                       seed = 7,
                       init = gen_init,
                       pars=c("Y_mean", "Y_sd"))

filename <- paste("SamplingResults\\RawSamples_betaPar1.dat", sep = "")
save(stanData, rawSamples, file = filename)

stanData$betaPar[1] <- 2.5

rm("filename", "rawSamples")


##
## Change betaPar[2]
## 

stanData$betaPar[2] <- 2.4

rawSamples <- sampling(BMNUS_sm, data = stanData, chains = nChains,
                       iter = 2000, warmup = 500,
                       refresh = 100,
                       seed = 7,
                       init = gen_init,
                       pars=c("Y_mean", "Y_sd"))

filename <- paste("SamplingResults\\RawSamples_betaPar2.dat", sep = "")
save(stanData, rawSamples, file = filename)

stanData$betaPar[2] <- 1.2

rm("filename", "rawSamples")

##
## Change gammaPar[1]
## 

stanData$gammaPar[1] <- 4.0

rawSamples <- sampling(BMNUS_sm, data = stanData, chains = nChains,
                       iter = 2000, warmup = 500,
                       refresh = 100,
                       seed = 7,
                       init = gen_init,
                       pars=c("Y_mean", "Y_sd"))

filename <- paste("SamplingResults\\RawSamples_gammaPar1.dat", sep = "")
save(stanData, rawSamples, file = filename)

stanData$gammaPar[1] <- 2.0

rm("filename", "rawSamples")

##
## Change gammaPar[2]
## 

stanData$gammaPar[2] <- 0.6

rawSamples <- sampling(BMNUS_sm, data = stanData, chains = nChains,
                       iter = 2000, warmup = 500,
                       refresh = 100,
                       seed = 7,
                       init = gen_init,
                       pars=c("Y_mean", "Y_sd"))

filename <- paste("SamplingResults\\RawSamples_gammaPar2.dat", sep = "")
save(stanData, rawSamples, file = filename)

stanData$gammaPar[2] <- 0.3

rm("filename", "rawSamples")

##
## Change cauchyPar
## 

stanData$cauchyPar <- 6

rawSamples <- sampling(BMNUS_sm, data = stanData, chains = nChains,
                       iter = 2000, warmup = 500,
                       refresh = 100,
                       seed = 7,
                       init = gen_init,
                       pars=c("Y_mean", "Y_sd"))

filename <- paste("SamplingResults\\RawSamples_cauchyPar.dat", sep = "")
save(stanData, rawSamples, file = filename)

stanData$gammaPar[2] <- 3

rm("filename", "rawSamples")

# Cleanup

options(oldPar)

rm("oldPar", "BMNUS_sm", "stanData", "nChains", "gen_init")

