##
## This file containes the R scripts that are used for the sensitivity analysis.
##

library(BMNUS)
library(tibble)
library(ggplot2)
library(MappingUtilities)
library(rstan)

##
## Determine the sample (field) points at which the sensitivity analysis is 
## performed and map them. The sample points are chosen to be near the 
## centers of the basis functions. The only reason for this choice is that
## approximately the same locations (i.e., almost exactly the same locations) 
## can be used for the analysis of the covariance function.
##

load("BasisFunctions\\Spacing_030.dat")

df <- as_tibble(basis_functions$centers_longlat)[c(293, 110, 260, 456), ]
df$label <- c("1", "2", "3", "4")

FindClosestPoint <- function(reference_location, locations){
  d2 <- (reference_location$long - locations$long)^2 +
    (reference_location$lat - locations$lat)^2
  
  return(which.min(d2))
}

index <- c(
  FindClosestPoint(df[1, ], Example_dataset),
  FindClosestPoint(df[2, ], Example_dataset),
  FindClosestPoint(df[3, ], Example_dataset),
  FindClosestPoint(df[4, ], Example_dataset))

reference_points <- as_tibble(cbind(label = 1:4,
                                    index,
                                    Example_dataset[index, c("long", "lat")]))

Add_Geography <- function(boundary_color = "gray70") {
  
  map_df <- ggplot2::map_data("state", region = c("Virginia", "North Carolina",
                                                  "South Carolina", "Georgia",
                                                  "Alabama", "Florida"))
  
  text_df <- data.frame(x = c(-86.7, -84, -82, -81, -80, -81.7, -78, - 86),
                        y = c(34.2, 34.2, 34.8, 35.9, 37, 28.5, 31, 28.5),
                        label = c("Alabama", "Georgia",
                                  "South\nCarolina", "North\nCarolina",
                                  "Virginia", "Florida", "Atlantic Ocean",
                                  "Gulf of Mexico"))
  return(
    list(ggplot2::geom_path(ggplot2::aes(x = long, y = lat, group = group),
                            data = map_df,
                            colour = boundary_color),
         ggplot2::geom_text(ggplot2::aes(x = x, y = y, label = label),
                            size = 2.5,
                            data = text_df)))
}

postscript("..\\Figures\\Figure_2-1.ps", width = 4, height = 4)
ggplot() +
  Add_Geography(boundary_color = "gray50") +
  Add_Path(Example_domain, "red") +
  Add_Points(reference_points, shape = 20, fill = "black", size = 3) +
  geom_label(aes(x = long, y = lat, label = label),
             data = reference_points,
             vjust = "top", color = "blue", nudge_y = -0.2) +
  Refine_Map("lambert", 25, 40, latLimits = c(28, 39.5))
dev.off()

# ----

# Index refers to the index of the sample that is used for the sensivity 
# analysis.
print(reference_points)


parameters <- c("None", "p1", "p2", "q1", "q2", "s")
suffixes <- c("", "_betaPar1", "_betaPar2", "_gammaPar1", "_gammaPar2", "_cauchyPar")
y_mean <- vector(mode = "list", length = 4)
y_sd <- vector(mode = "list", length = 4)
for(i in 1:6){
  file <- paste0("SamplingResults\\RawSamples", suffixes[i], ".dat")
  print(file)
  load(file)
  postSamples <- extract(rawSamples)

  y_mean[[1]] <- rbind(y_mean[[1]],
                  data.frame(value = postSamples$Y_mean[, reference_points$index[[1]]],
                             parameter = parameters[i],
                             stringsAsFactors = FALSE))
  y_sd[[1]] <- rbind(y_sd[[1]],
                data.frame(value = postSamples$Y_sd[, reference_points$index[[1]]],
                           parameter = parameters[i],
                           stringsAsFactors = FALSE))
  
  y_mean[[2]] <- rbind(y_mean[[2]],
                       data.frame(value = postSamples$Y_mean[, reference_points$index[[2]]],
                                  parameter = parameters[i],
                                  stringsAsFactors = FALSE))
  y_sd[[2]] <- rbind(y_sd[[2]],
                     data.frame(value = postSamples$Y_sd[, reference_points$index[[2]]],
                                parameter = parameters[i],
                                stringsAsFactors = FALSE))

  y_mean[[3]] <- rbind(y_mean[[3]],
                       data.frame(value = postSamples$Y_mean[, reference_points$index[[3]]],
                                  parameter = parameters[i],
                                  stringsAsFactors = FALSE))
  y_sd[[3]] <- rbind(y_sd[[3]],
                     data.frame(value = postSamples$Y_sd[, reference_points$index[[3]]],
                                parameter = parameters[i],
                                stringsAsFactors = FALSE))

  y_mean[[4]] <- rbind(y_mean[[4]],
                       data.frame(value = postSamples$Y_mean[, reference_points$index[[4]]],
                                  parameter = parameters[i],
                                  stringsAsFactors = FALSE))
  y_sd[[4]] <- rbind(y_sd[[4]],
                     data.frame(value = postSamples$Y_sd[, reference_points$index[[4]]],
                                parameter = parameters[i],
                                stringsAsFactors = FALSE))
}

PlotSensAnalysis <- function(y_mean, y_sd) {
  p <- ggplot(y_mean, aes(x = parameter, y = value)) +
    # geom_boxplot() +
    geom_violin() +
    scale_x_discrete(name = "Constant") +
    scale_y_continuous(name = "\nProcess mean") +
    ggtitle("A.") +
    theme_bw() +
    theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank()) +
    theme(plot.title=element_text(hjust = 0, size = 10, face = "bold"))
  
  q <- ggplot(y_sd, aes(x = parameter, y = value)) +
    # geom_boxplot() +
    geom_violin() +
    scale_x_discrete(name = "Constant") +
    scale_y_continuous(name = "Process\nstandard deviation") +
    ggtitle("B.") +
    theme_bw() +
    theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank()) +
    theme(plot.title=element_text(hjust = 0, size = 10, face = "bold"))
  
  grid::grid.newpage()
  grid::pushViewport(grid::viewport(layout=grid::grid.layout(2,1)))
  plot(p, vp=grid::viewport(layout.pos.row=1, layout.pos.col=1))
  plot(q, vp=grid::viewport(layout.pos.row=2, layout.pos.col=1))
  
}

postscript("..\\Figures\\Figure_2-2.ps", width = 6.5, height = 3.5)
PlotSensAnalysis(y_mean[[1]], y_sd[[1]])
dev.off()

postscript("..\\Figures\\Figure_2-3.ps", width = 6.5, height = 3.5)
PlotSensAnalysis(y_mean[[2]], y_sd[[2]])
dev.off()

postscript("..\\Figures\\Figure_2-4.ps", width = 6.5, height = 3.5)
PlotSensAnalysis(y_mean[[3]], y_sd[[3]])
dev.off()

postscript("..\\Figures\\Figure_2-5.ps", width = 6.5, height = 3.5)
PlotSensAnalysis(y_mean[[4]], y_sd[[4]])
dev.off()

